import React, { createContext, useState, useContext, useEffect } from 'react';
import { useColorScheme } from 'react-native';
import { defaultColors } from './colors';
const defaultSizes = {
  screenPadding: 16,
  buttonHeight: 54,
  searchHeight: 54,
  chipsItemHeight: 44,
  chipsItemBorderRadius: 12,
  chipsItemGapSize: 6,
  chipsItemPaddingLeft: 12,
  chipsItemPaddingRight: 12,
  checkboxSize: 20,
  checkboxBorderRadius: 4
};

const getInitialConfigs = function (props) {
  let scheme = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'light';
  const baseColors = { ...defaultColors[scheme],
    ...(props === null || props === void 0 ? void 0 : props.colors)
  };
  const sizes = { ...defaultSizes,
    ...(props === null || props === void 0 ? void 0 : props.sizes)
  };
  const defaultTextStyle = [{
    color: baseColors.textDefaultColor
  }, props === null || props === void 0 ? void 0 : props.textBaseStyle];
  return {
    colors: baseColors,
    sizes: sizes,
    textBaseStyle: defaultTextStyle,
    textLinkStyle: [...defaultTextStyle, {
      color: baseColors.textLinkColor
    }, props === null || props === void 0 ? void 0 : props.textLinkStyle],
    buttonContainerStyle: [{}, props === null || props === void 0 ? void 0 : props.buttonContainerStyle],
    buttonStyle: [{
      minHeight: sizes.buttonHeight,
      flexDirection: 'row',
      alignItems: 'center',
      justifyContent: 'space-between',
      backgroundColor: baseColors.buttonBackgroundColor,
      borderWidth: 1,
      borderColor: baseColors.buttonBorderColor,
      borderRadius: 10,
      paddingHorizontal: 16,
      paddingVertical: 5
    }, props === null || props === void 0 ? void 0 : props.buttonStyle],
    buttonLabelStyle: [...defaultTextStyle, {
      color: baseColors.buttonLabelColor,
      fontWeight: 'bold',
      marginVertical: 4
    }, props === null || props === void 0 ? void 0 : props.buttonLabelStyle],
    buttonTextStyle: [...defaultTextStyle, {
      color: baseColors.buttonTextColor,
      fontWeight: 'bold',
      flex: 1
    }, props === null || props === void 0 ? void 0 : props.buttonTextStyle],
    buttonCaretImageStyle: [{
      width: 20,
      height: 20,
      tintColor: baseColors.buttonCaretColor
    }, props === null || props === void 0 ? void 0 : props.buttonCaretImageStyle],
    containerStyle: [{
      backgroundColor: baseColors.backgroundColor
    }, props === null || props === void 0 ? void 0 : props.containerStyle],
    chipsGridStyle: [{
      flexDirection: 'row',
      alignItems: 'center',
      flexWrap: 'wrap',
      marginLeft: -sizes.chipsItemGapSize / 2,
      marginRight: -sizes.chipsItemGapSize / 2
    }, props === null || props === void 0 ? void 0 : props.chipsGridStyle],
    chipsItemContainerStyle: [{
      flexDirection: 'row',
      justifyContent: 'center',
      alignItems: 'center',
      minHeight: sizes.chipsItemHeight,
      borderRadius: sizes.chipsItemBorderRadius,
      margin: sizes.chipsItemGapSize / 2,
      paddingLeft: sizes.chipsItemPaddingLeft,
      paddingRight: sizes.chipsItemPaddingRight,
      backgroundColor: baseColors.chipsItemBackgroundColor,
      borderWidth: 1,
      borderColor: baseColors.chipsItemBorderColor
    }, props === null || props === void 0 ? void 0 : props.chipsItemContainerStyle],
    chipsItemCloseButtonStyle: [{
      backgroundColor: baseColors.chipsItemCloseButtonBackgroundColor,
      justifyContent: 'center',
      alignItems: 'center',
      marginLeft: 8,
      aspectRatio: 1 / 1,
      padding: 8,
      borderRadius: 99
    }, props === null || props === void 0 ? void 0 : props.chipsItemCloseButtonStyle],
    chipsItemCloseIconStyle: [{
      width: 16,
      height: 16,
      tintColor: baseColors.chipsItemCloseButtonIconColor
    }, props === null || props === void 0 ? void 0 : props.chipsItemCloseIconStyle],
    chipsItemTextStyle: [...defaultTextStyle, {
      fontWeight: 'bold'
    }, props === null || props === void 0 ? void 0 : props.chipsItemCloseIconStyle],
    headerContainerStyle: [{
      height: 54,
      flexDirection: 'row',
      alignItems: 'center',
      paddingHorizontal: 16,
      borderBottomWidth: 1,
      borderBottomColor: baseColors.borderColor
    }, props === null || props === void 0 ? void 0 : props.headerContainerStyle],
    headerTextStyle: [...defaultTextStyle, {}, props === null || props === void 0 ? void 0 : props.headerTextStyle],
    headerTitleStyle: [...defaultTextStyle, {
      textAlign: 'center',
      fontWeight: 'bold'
    }, props === null || props === void 0 ? void 0 : props.headerTitleStyle],
    searchInputStyle: [...defaultTextStyle, {
      height: sizes.searchHeight,
      margin: sizes.screenPadding,
      borderWidth: 1,
      borderColor: baseColors.borderColor,
      borderRadius: sizes.searchHeight / 2,
      paddingHorizontal: 16
    }, props === null || props === void 0 ? void 0 : props.searchInputStyle],
    pickerItemContainerStyle: [{
      flexDirection: 'row',
      alignItems: 'center',
      padding: 16,
      opacity: 1
    }, props === null || props === void 0 ? void 0 : props.pickerItemContainerStyle],
    pickerItemTextStyle: [...defaultTextStyle, {
      flex: 1
    }, props === null || props === void 0 ? void 0 : props.pickerItemTextStyle],
    pickerItemIconStyle: [{
      width: 24,
      height: 24,
      marginRight: 8,
      tintColor: baseColors.chipsItemCloseButtonIconColor
    }, props === null || props === void 0 ? void 0 : props.pickerItemIconStyle],
    pickerItemActiveIconStyle: [{
      tintColor: baseColors.textDefaultColor
    }, props === null || props === void 0 ? void 0 : props.pickerItemActiveIconStyle],
    pickerItemDisabledStyle: [{
      opacity: 0.5
    }, props === null || props === void 0 ? void 0 : props.pickerItemDisabledStyle],
    resultTextStyle: [...defaultTextStyle, props === null || props === void 0 ? void 0 : props.resultTextStyle]
  };
};

const AsyncPickerContext = /*#__PURE__*/createContext(undefined);

const AsyncPickerProvider = _ref => {
  let {
    children,
    configParams = undefined
  } = _ref;
  const colorScheme = useColorScheme() === 'dark' ? 'dark' : 'light';
  const [configs, setConfigs] = useState(getInitialConfigs(configParams));
  useEffect(() => {
    setConfigs(getInitialConfigs(configParams, colorScheme));
  }, [configParams, colorScheme]);
  return /*#__PURE__*/React.createElement(AsyncPickerContext.Provider, {
    value: {
      configs,
      setConfigs
    }
  }, children);
};

function useAsyncPicker() {
  const context = useContext(AsyncPickerContext);

  if (context === undefined) {
    throw new Error('useCount must be used within a CountProvider');
  }

  return context;
}

export { AsyncPickerProvider, useAsyncPicker };
//# sourceMappingURL=AsyncPickerContext.js.map